searchData={"items":[{"type":"module","title":"Redix","doc":"This module provides the main API to interface with [Redis](http://redis.io) and\n[Valkey](https://valkey.io/).","ref":"Redix.html"},{"type":"module","title":"Overview - Redix","doc":"`start_link/2` starts a process that connects to Redis. Each Elixir process\nstarted with this function maps to a client TCP connection to the specified\nRedis server.\n\nThe architecture is very simple: when you issue commands to Redis (via\n`command/3` or `pipeline/3`), the Redix process sends the command to Redis right\naway and is immediately able to send new commands. When a response arrives\nfrom Redis, only then the Redix process replies to the caller with the\nresponse. This pattern avoids blocking the Redix process for each request (until\na response arrives), increasing the performance of this driver.","ref":"Redix.html#module-overview"},{"type":"module","title":"Reconnections - Redix","doc":"Redix tries to be as resilient as possible: it tries to recover automatically\nfrom most network errors.\n\nIf there's a network error when sending data to Redis or if the connection to Redis\ndrops, Redix tries to reconnect. The first reconnection attempt will happen\nafter a fixed time interval; if this attempt fails, reconnections are\nattempted until successful, and the time interval between reconnections is\nincreased exponentially. Some aspects of this behaviour can be configured; see\n`start_link/2` and the \"Reconnections\" page in the docs for more information.","ref":"Redix.html#module-reconnections"},{"type":"module","title":"Sentinel - Redix","doc":"Redix supports [Redis Sentinel](https://redis.io/topics/sentinel) by passing a `:sentinel`\noption to `start_link/1` (or `start_link/2`) instead of `:host` and `:port`. In `:sentinel`,\nyou'll specify a list of sentinel nodes to try when connecting and the name of a primary group\n(see `start_link/1` for more detailed information on these options). When connecting, Redix will\nattempt to connect to each of the specified sentinels in the given order. When it manages to\nconnect to a sentinel, it will ask that sentinel for the address of the primary for the given\nprimary group. Then, it will connect to that primary and ask it for confirmation that it is\nindeed a primary. If anything in this process doesn't go right, the next sentinel in the list\nwill be tried.\n\nAll of this happens in case of disconnections as well. If there's a disconnection, the whole\nprocess of asking sentinels for a primary is executed again.\n\nYou should only care about Redis Sentinel when starting a `Redix` connection: once started,\nusing the connection will be exactly the same as the non-sentinel scenario.","ref":"Redix.html#module-sentinel"},{"type":"module","title":"Transactions or pipelining? - Redix","doc":"Pipelining and transactions have things in common but they're fundamentally different.\nWith a pipeline, you're sending all commands in the pipeline *at once* on the connection\nto Redis. This means Redis receives all commands at once, but the Redis server is not\nguaranteed to process all those commands at once.\n\nOn the other hand, a `MULTI`/`EXEC` transaction guarantees that when `EXEC` is called\nall the queued commands in the transaction are executed atomically. However, you don't\nneed to send all the commands in the transaction at once. If you want to combine\npipelining with `MULTI`/`EXEC` transactions, use `transaction_pipeline/3`.","ref":"Redix.html#module-transactions-or-pipelining"},{"type":"module","title":"Skipping replies - Redix","doc":"Redis provides commands to control whether you want replies to your commands or not.\nThese commands are `CLIENT REPLY ON`, `CLIENT REPLY SKIP`, and `CLIENT REPLY OFF`.\nWhen you use `CLIENT REPLY SKIP`, only the command that follows will not get a reply.\nWhen you use `CLIENT REPLY OFF`, all the commands that follow will not get replies until\n`CLIENT REPLY ON` is issued. Redix does not support these commands directly because they\nwould change the whole state of the connection. To skip replies, use `noreply_pipeline/3`\nor `noreply_command/3`.\n\nSkipping replies is useful to improve performance when you want to issue many commands\nbut are not interested in the responses to those commands.\n\n> #","ref":"Redix.html#module-skipping-replies"},{"type":"module","title":"Blocked `CLIENT` commands {: .warning} - Redix","doc":">\n> Some servers may block `CLIENT` commands. For example, Google Memorystorage is [does\n> this](https://cloud.google.com/memorystore/docs/redis/product-constraints).\n> If this is the case, the `noreply_*` functions mentioned above won't work.","ref":"Redix.html#module-blocked-client-commands-warning"},{"type":"module","title":"SSL - Redix","doc":"Redix supports SSL by passing `ssl: true` in `start_link/1`. You can use the `:socket_opts`\noption to pass options that will be used by the SSL socket, like certificates.\n\nIf the [CAStore](https://hex.pm/packages/castore) dependency is available, Redix will pick\nup its CA certificate store file automatically. You can select a different CA certificate\nstore by passing in the `:cacertfile` or `:cacerts` socket options. If the server uses a\nself-signed certificate, such as for testing purposes, disable certificate verification by\npassing `verify: :verify_none` in the socket options.\n\nSome Redis servers, notably Amazon ElastiCache, use wildcard certificates that require\nadditional socket options for successful verification (requires OTP 21.0 or later):\n\n    Redix.start_link(\n      host: \"example.com\", port: 9999, ssl: true,\n      socket_opts: [\n        customize_hostname_check: [\n          match_fun: :public_key.pkix_verify_hostname_match_fun(:https)\n        ]\n      ]\n    )","ref":"Redix.html#module-ssl"},{"type":"module","title":"Telemetry - Redix","doc":"Redix uses Telemetry for instrumentation and logging. See `Redix.Telemetry`.","ref":"Redix.html#module-telemetry"},{"type":"function","title":"Redix.child_spec/1","doc":"Returns a child spec to use Redix in supervision trees.\n\nTo use Redix with the default options (same as calling `Redix.start_link()`):\n\n    children = [\n      Redix,\n      # ...\n    ]\n\nYou can pass options:\n\n    children = [\n      {Redix, host: \"redix.example.com\", name: :redix},\n      # ...\n    ]\n\nYou can also pass a URI:\n\n    children = [\n      {Redix, \"redis://redix.example.com:6380\"}\n    ]\n\nIf you want to pass both a URI and options, you can do it by passing a tuple with the URI as the\nfirst element and the list of options (make sure it has brackets around if using literals) as\nthe second element:\n\n    children = [\n      {Redix, {\"redis://redix.example.com\", [name: :redix]}}\n    ]","ref":"Redix.html#child_spec/1"},{"type":"function","title":"Redix.command/3","doc":"Issues a command on the Redis server.\n\nThis function sends `command` to the Redis server and returns the response\nreturned by Redis. `pid` must be the pid of a Redix connection. `command` must\nbe a list of strings making up the Redis command and its arguments.\n\nThe return value is `{:ok, response}` if the request is successful and the\nresponse is not a Redis error. `{:error, reason}` is returned in case there's\nan error in the request (such as losing the connection to Redis in between the\nrequest). `reason` can also be a `Redix.Error` exception in case Redis is\nreachable but returns an error (such as a type error).\n\nIf the given command is an empty command (`[]`), an `ArgumentError`\nexception is raised.\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#command/3"},{"type":"function","title":"Examples - Redix.command/3","doc":"iex> Redix.command(conn, [\"SET\", \"mykey\", \"foo\"])\n    {:ok, \"OK\"}\n    iex> Redix.command(conn, [\"GET\", \"mykey\"])\n    {:ok, \"foo\"}\n\n    iex> Redix.command(conn, [\"INCR\", \"mykey\"])\n    {:error, \"ERR value is not an integer or out of range\"}\n\nIf Redis goes down (before a reconnection happens):\n\n    iex> {:error, error} = Redix.command(conn, [\"GET\", \"mykey\"])\n    iex> error.reason\n    :closed","ref":"Redix.html#command/3-examples"},{"type":"function","title":"Redix.command!/3","doc":"Issues a command on the Redis server, raising if there's an error.\n\nThis function works exactly like `command/3` but:\n\n  * if the command is successful, then the result is returned directly (not wrapped in a\n    `{:ok, result}` tuple).\n  * if there's a Redis error or a connection error, a `Redix.Error` or `Redix.ConnectionError`\n    error is raised.\n\nThis function accepts the same options as `command/3`.","ref":"Redix.html#command!/3"},{"type":"function","title":"Examples - Redix.command!/3","doc":"iex> Redix.command!(conn, [\"SET\", \"mykey\", \"foo\"])\n    \"OK\"\n\n    iex> Redix.command!(conn, [\"INCR\", \"mykey\"])\n    ** (Redix.Error) ERR value is not an integer or out of range\n\nIf Redis goes down (before a reconnection happens):\n\n    iex> Redix.command!(conn, [\"GET\", \"mykey\"])\n    ** (Redix.ConnectionError) :closed","ref":"Redix.html#command!/3-examples"},{"type":"function","title":"Redix.noreply_command/3","doc":"Same as `command/3` but tells the Redis server to not return a response.\n\nThis function is useful when you want to send a command but you don't care about the response.\nSince the response is not returned, the return value of this function in case the command\nis successfully sent to Redis is `:ok`.\n\nNot receiving a response means saving traffic on the network and memory allocation for the\nresponse. See also `noreply_pipeline/3`.\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#noreply_command/3"},{"type":"function","title":"Examples - Redix.noreply_command/3","doc":"iex> Redix.noreply_command(conn, [\"INCR\", \"mykey\"])\n    :ok\n    iex> Redix.command(conn, [\"GET\", \"mykey\"])\n    {:ok, \"1\"}","ref":"Redix.html#noreply_command/3-examples"},{"type":"function","title":"Redix.noreply_command!/3","doc":"Same as `noreply_command/3` but raises in case of errors.","ref":"Redix.html#noreply_command!/3"},{"type":"function","title":"Redix.noreply_pipeline/3","doc":"Issues a pipeline of commands to the Redis server, asking the server to not send responses.\n\nThis function is useful when you want to issue commands to the Redis server but you don't\ncare about the responses. For example, you might want to set a bunch of keys but you don't\ncare for a confirmation that they were set. In these cases, you can save bandwidth by asking\nRedis to not send replies to your commands.\n\nSince no replies are sent back, this function returns `:ok` in case there are no network\nerrors, or `{:error, reason}` otherwise.any()\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#noreply_pipeline/3"},{"type":"function","title":"Examples - Redix.noreply_pipeline/3","doc":"iex> commands = [[\"INCR\", \"mykey\"], [\"INCR\", \"meykey\"]]\n    iex> Redix.noreply_pipeline(conn, commands)\n    :ok\n    iex> Redix.command(conn, [\"GET\", \"mykey\"])\n    {:ok, \"2\"}","ref":"Redix.html#noreply_pipeline/3-examples"},{"type":"function","title":"Redix.noreply_pipeline!/3","doc":"Same as `noreply_pipeline/3` but raises in case of errors.","ref":"Redix.html#noreply_pipeline!/3"},{"type":"function","title":"Redix.pipeline/3","doc":"Issues a pipeline of commands on the Redis server.\n\n`commands` must be a list of commands, where each command is a list of strings\nmaking up the command and its arguments. The commands will be sent as a single\n\"block\" to Redis, and a list of ordered responses (one for each command) will\nbe returned.\n\nThe return value is `{:ok, results}` if the request is successful, `{:error,\nreason}` otherwise.\n\nNote that `{:ok, results}` is returned even if `results` contains one or more\nRedis errors (`Redix.Error` structs). This is done to avoid having to walk the\nlist of results (a `O(n)` operation) to look for errors, leaving the\nresponsibility to the user. That said, errors other than Redis errors (like\nnetwork errors) always cause the return value to be `{:error, reason}`.\n\nIf `commands` is an empty list (`[]`) or any of the commands in `commands` is\nan empty command (`[]`) then an `ArgumentError` exception is raised right\naway.\n\nPipelining is not the same as a transaction. For more information, see the\nmodule documentation.","ref":"Redix.html#pipeline/3"},{"type":"function","title":"Options - Redix.pipeline/3","doc":"* `:timeout` (`t:timeout/0`) - request timeout (in milliseconds). If the Redis server doesn't reply within this timeout,\n  `{:error, %Redix.ConnectionError{reason: :timeout}}` is returned. The default value is `5000`.\n\n* `:telemetry_metadata` (map of `t:term/0` keys and `t:term/0` values) - extra metadata to add to the `[:redix, :pipeline, *]` Telemetry events.\n  These end up in the `:extra_metadata` metadata key of these events. See `Redix.Telemetry`. The default value is `%{}`.","ref":"Redix.html#pipeline/3-options"},{"type":"function","title":"Examples - Redix.pipeline/3","doc":"iex> Redix.pipeline(conn, [[\"INCR\", \"mykey\"], [\"INCR\", \"mykey\"], [\"DECR\", \"mykey\"]])\n    {:ok, [1, 2, 1]}\n\n    iex> Redix.pipeline(conn, [[\"SET\", \"k\", \"foo\"], [\"INCR\", \"k\"], [\"GET\", \"k\"]])\n    {:ok, [\"OK\", %Redix.Error{message: \"ERR value is not an integer or out of range\"}, \"foo\"]}\n\nIf Redis goes down (before a reconnection happens):\n\n    iex> {:error, error} = Redix.pipeline(conn, [[\"SET\", \"mykey\", \"foo\"], [\"GET\", \"mykey\"]])\n    iex> error.reason\n    :closed\n\nExtra Telemetry metadata:\n\n    iex> Redix.pipeline(conn, [[\"PING\"]], telemetry_metadata: %{connection: \"My conn\"})","ref":"Redix.html#pipeline/3-examples"},{"type":"function","title":"Redix.pipeline!/3","doc":"Issues a pipeline of commands to the Redis server, raising if there's an error.\n\nThis function works similarly to `pipeline/3`, except:\n\n  * if there are no errors in issuing the commands (even if there are one or\n    more Redis errors in the results), the results are returned directly (not\n    wrapped in a `{:ok, results}` tuple).\n  * if there's a connection error then a `Redix.ConnectionError` exception is raised.\n\nFor more information on why nothing is raised if there are one or more Redis\nerrors (`Redix.Error` structs) in the list of results, look at the\ndocumentation for `pipeline/3`.\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#pipeline!/3"},{"type":"function","title":"Examples - Redix.pipeline!/3","doc":"iex> Redix.pipeline!(conn, [[\"INCR\", \"mykey\"], [\"INCR\", \"mykey\"], [\"DECR\", \"mykey\"]])\n    [1, 2, 1]\n\n    iex> Redix.pipeline!(conn, [[\"SET\", \"k\", \"foo\"], [\"INCR\", \"k\"], [\"GET\", \"k\"]])\n    [\"OK\", %Redix.Error{message: \"ERR value is not an integer or out of range\"}, \"foo\"]\n\nIf Redis goes down (before a reconnection happens):\n\n    iex> Redix.pipeline!(conn, [[\"SET\", \"mykey\", \"foo\"], [\"GET\", \"mykey\"]])\n    ** (Redix.ConnectionError) :closed","ref":"Redix.html#pipeline!/3-examples"},{"type":"function","title":"Redix.start_link/1","doc":"Starts a connection to Redis.\n\nThis function returns `{:ok, pid}` if the Redix process is started\nsuccessfully.\n\n    {:ok, pid} = Redix.start_link()\n\nThe actual TCP connection to the Redis server may happen either synchronously,\nbefore `start_link/2` returns, or asynchronously. This behaviour is decided by\nthe `:sync_connect` option (see below).\n\nThis function accepts one argument which can either be an string representing\na URI or a keyword list of options.","ref":"Redix.html#start_link/1"},{"type":"function","title":"Using in supervision trees - Redix.start_link/1","doc":"Redix supports child specs, so you can use it as part of a supervision tree:\n\n    children = [\n      {Redix, host: \"redix.myapp.com\", name: :redix}\n    ]\n\nSee `child_spec/1` for more information.","ref":"Redix.html#start_link/1-using-in-supervision-trees"},{"type":"function","title":"Using a Redis URI - Redix.start_link/1","doc":"In case `uri_or_options` is a Redis URI, it must be in the form:\n\n    redis://[username:password@]host[:port][/db]\n\nHere are some examples of valid URIs:\n\n  * `redis://localhost`\n  * `redis://:secret@localhost:6397`\n  * `redis://username:secret@localhost:6397`\n  * `redis://example.com:6380/1`\n  * `rediss://example.com:6380/1` (for SSL connections)\n  * `valkey://example.com:6380/1` (for [Valkey](https://valkey.io/) connections)\n\nThe only mandatory thing when using URIs is the host. All other elements are optional\nand their default value can be found in the \"Options\" section below.\n\nIn earlier versions of Redix, the username in the URI was ignored. Redis 6 introduced [ACL\nsupport](https://redis.io/topics/acl). Now, Redix supports usernames as well.\n\n> ##","ref":"Redix.html#start_link/1-using-a-redis-uri"},{"type":"function","title":"Valkey {: .info} - Redix.start_link/1","doc":">\n> The `valkey://` schema is supported since Redix v1.5.0.","ref":"Redix.html#start_link/1-valkey-info"},{"type":"function","title":"Options - Redix.start_link/1","doc":"The following options can be used to specify the connection:\n\n* `:host` (`t:String.t/0`) - the host where the Redis server is running. If you are using a Redis URI, you cannot\n  use this option. Defaults to `\"localhost`\".\n\n* `:port` (`t:non_neg_integer/0`) - the port on which the Redis server is running. If you are using a Redis URI, you cannot\n  use this option. Defaults to `6379`.\n\n* `:database` (`t:String.t/0` or `t:non_neg_integer/0`) - the database to connect to. Defaults to `nil`, meaning Redix doesn't connect to a\n  specific database (the default in this case is database `0`). When this option is provided,\n  all Redix does is issue a `SELECT` command to Redis in order to select the given database.\n\n* `:username` - the username to connect to Redis. Defaults to `nil`, meaning no username is used.\n  Redis supports usernames only since Redis 6 (see the [ACL\n  documentation](https://redis.io/topics/acl)). If a username is provided (either via\n  options or via URIs) and the Redis version used doesn't support ACL, then Redix falls\n  back to using just the password and emits a warning. In future Redix versions, Redix\n  will raise if a username is passed and the Redis version used doesn't support ACL.\n\n* `:password` (`t:Redix.password/0`) - the password used to connect to Redis. Defaults to\n  `nil`, meaning no password is used. When this option is provided, all Redix\n  does is issue an `AUTH` command to Redis in order to authenticate. MFAs are also\n  supported in the form of `{module, function, arguments}`. This can be used\n  to fetch the password dynamically on every reconnection but most importantly to\n  hide the password from crash reports in case the Redix connection crashes for\n  any reason. For example, you can set this option to:\n  `{System, :fetch_env!, [\"REDIX_PASSWORD\"]}`.\n\n* `:timeout` (`t:timeout/0`) - connection timeout (in milliseconds) directly passed to the network layer. The default value is `5000`.\n\n* `:sync_connect` (`t:boolean/0`) - decides whether Redix should initiate the network connection to the Redis server *before*\n  or *after* returning from `start_link/1`. This option also changes some reconnection\n  semantics; read the \"Reconnections\" page in the documentation for more information. The default value is `false`.\n\n* `:exit_on_disconnection` (`t:boolean/0`) - if `true`, the Redix server will exit if it fails to connect or disconnects from Redis.\n  Note that setting this option to `true` means that the `:backoff_initial` and\n  `:backoff_max` options will be ignored. The default value is `false`.\n\n* `:backoff_initial` (`t:non_neg_integer/0`) - the initial backoff time (in milliseconds), which is the time that the Redix process\n  will wait before attempting to reconnect to Redis after a disconnection or failed first\n  connection. See the \"Reconnections\" page in the docs for more information. The default value is `500`.\n\n* `:backoff_max` (`t:timeout/0`) - the maximum length (in milliseconds) of the time interval used between reconnection\n  attempts. See the \"Reconnections\" page in the docs for more information. The default value is `30000`.\n\n* `:ssl` (`t:boolean/0`) - if `true`, connect through SSL, otherwise through TCP. The `:socket_opts` option applies\n  to both SSL and TCP, so it can be used for things like certificates. See `:ssl.connect/4`. The default value is `false`.\n\n* `:name` (`t:term/0`) - Redix is bound to the same registration rules as a `GenServer`. See the `GenServer`\n  documentation for more information.\n\n* `:socket_opts` (list of `t:term/0`) - specifies a list of options that are passed to the network layer when connecting to\n  the Redis server. Some socket options (like `:active` or `:binary`) will be\n  overridden by Redix so that it functions properly.\n\n  If `ssl: true`, then these are added to the default: `[verify: :verify_peer, depth: 3]`.\n  If the `CAStore` dependency is available, the `:cacertfile` option is added\n  to the SSL options by default as well.\n\n  The default value is `[]`.\n\n* `:hibernate_after` (`t:non_neg_integer/0`) - if present, the Redix connection process awaits any message for the given number\n  of milliseconds and if no message is received, the process goes into hibernation\n  automatically (by calling `:proc_lib.hibernate/3`). See `t::gen_statem.start_opt/0`.\n  Not present by default.\n\n* `:spawn_opt` (`t:keyword/0`) - if present, its value is passed as options to the Redix connection process as in\n  `Process.spawn/4`. See `t::gen_statem.start_opt/0`. Not present by default.\n\n* `:debug` (`t:keyword/0`) - if present, the corresponding function in the\n  [`:sys` module](http://www.erlang.org/doc/man/sys.html) is invoked.\n\n* `:sentinel` (`t:keyword/0`) - options to use Redis Sentinel. If this option is present, you cannot use the `:host` and\n  `:port` options. See the [*Sentinel Options* section below](#start_link/1-sentinel-options).\n\n#","ref":"Redix.html#start_link/1-options"},{"type":"function","title":"Sentinel Options - Redix.start_link/1","doc":"* `:sentinels` (list of `t:String.t/0` or `t:keyword/0`) - Required. a list of sentinel addresses. Each element in this list is the address\n  of a sentinel to be contacted in order to obtain the address of a primary. The address of\n  a sentinel can be passed as a Redis URI (see the \"Using a Redis URI\" section) or\n  a keyword list with `:host`, `:port`, `:password` options (same as when connecting to a\n  Redis instance directly). Note that the password can either be passed in the sentinel\n  address or globally — see the `:password` option below.\n\n* `:group` (`t:String.t/0`) - Required. the name of the group that identifies the primary in the sentinel configuration.\n\n* `:role` (`t:Redix.sentinel_role/0`) - if `:primary`, the connection will be established\n  with the primary for the given group. If `:replica`, Redix will ask the sentinel for all\n  the available replicas for the given group and try to connect to one of them\n  **at random**. The default value is `:primary`.\n\n* `:socket_opts` (`t:keyword/0`) - socket options for connecting to each sentinel. Same as the `:socket_opts` option\n  described above. The default value is `[]`.\n\n* `:timeout` (`t:timeout/0`) - the timeout (in milliseconds or `:infinity`) that will be used to\n  interact with the sentinels. This timeout will be used as the timeout when connecting to\n  each sentinel and when asking sentinels for a primary. The Redis documentation suggests\n  to keep this timeout short so that connection to Redis can happen quickly. The default value is `500`.\n\n* `:ssl` (`t:boolean/0`) - whether to use SSL to connect to each sentinel. The default value is `false`.\n\n* `:password` (`t:Redix.password/0`) - if you don't want to specify a password for each sentinel you\n  list, you can use this option to specify a password that will be used to authenticate\n  on sentinels if they don't specify a password. This option is recommended over passing\n  a password for each sentinel because in the future we might do sentinel auto-discovery,\n  which means authentication can only be done through a global password that works for all\n  sentinels.","ref":"Redix.html#start_link/1-sentinel-options"},{"type":"function","title":"Examples - Redix.start_link/1","doc":"iex> Redix.start_link()\n    {:ok, #PID<...>}\n\n    iex> Redix.start_link(host: \"example.com\", port: 9999, password: \"secret\")\n    {:ok, #PID<...>}\n\n    iex> Redix.start_link(database: 3, name: :redix_3)\n    {:ok, #PID<...>}","ref":"Redix.html#start_link/1-examples"},{"type":"function","title":"Redix.start_link/2","doc":"Starts a connection to Redis.\n\nThis is the same as `start_link/1`, but the URI and the options get merged. `other_opts` have\nprecedence over the things specified in `uri`. Take this code:\n\n    Redix.start_link(\"redis://localhost:6379\", port: 6380)\n\nIn this example, port `6380` will be used.","ref":"Redix.html#start_link/2"},{"type":"function","title":"Redix.stop/2","doc":"Closes the connection to the Redis server.\n\nThis function is synchronous and blocks until the given Redix connection frees\nall its resources and disconnects from the Redis server. `timeout` can be\npassed to limit the amount of time allowed for the connection to exit; if it\ndoesn't exit in the given interval, this call exits.","ref":"Redix.html#stop/2"},{"type":"function","title":"Examples - Redix.stop/2","doc":"iex> Redix.stop(conn)\n    :ok","ref":"Redix.html#stop/2-examples"},{"type":"function","title":"Redix.transaction_pipeline/3","doc":"Executes a `MULTI`/`EXEC` transaction.\n\nRedis supports something akin to transactions. It works by sending a `MULTI` command,\nthen some commands, and then an `EXEC` command. All the commands after `MULTI` are\nqueued until `EXEC` is issued. When `EXEC` is issued, all the responses to the queued\ncommands are returned in a list.\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#transaction_pipeline/3"},{"type":"function","title":"Examples - Redix.transaction_pipeline/3","doc":"To run a `MULTI`/`EXEC` transaction in one go, use this function and pass a list of\ncommands to use in the transaction:\n\n    iex> Redix.transaction_pipeline(conn, [[\"SET\", \"mykey\", \"foo\"], [\"GET\", \"mykey\"]])\n    {:ok, [\"OK\", \"foo\"]}","ref":"Redix.html#transaction_pipeline/3-examples"},{"type":"function","title":"Problems with transactions - Redix.transaction_pipeline/3","doc":"There's an inherent problem with Redix's architecture and `MULTI`/`EXEC` transaction.\nA Redix process is a single connection to Redis that can be used by many clients. If\na client A sends `MULTI` and client B sends a command before client A sends `EXEC`,\nclient B's command will be part of the transaction. This is intended behaviour, but\nit might not be what you expect. This is why `transaction_pipeline/3` exists: this function\nwraps `commands` in `MULTI`/`EXEC` but *sends all in a pipeline*. Since everything\nis sent in the pipeline, it's sent at once on the connection and no commands can\nend up in the middle of the transaction.","ref":"Redix.html#transaction_pipeline/3-problems-with-transactions"},{"type":"function","title":"Running `MULTI`/`EXEC` transactions manually - Redix.transaction_pipeline/3","doc":"There are still some cases where you might want to start a transaction with `MULTI`,\nthen send commands from different processes that you actively want to be in the\ntransaction, and then send an `EXEC` to run the transaction. It's still fine to do\nthis with `command/3` or `pipeline/3`, but remember what explained in the section\nabove. If you do this, do it in an isolated connection (open a new one if necessary)\nto avoid mixing things up.","ref":"Redix.html#transaction_pipeline/3-running-multi-exec-transactions-manually"},{"type":"function","title":"Redix.transaction_pipeline!/3","doc":"Executes a `MULTI`/`EXEC` transaction.\n\nSame as `transaction_pipeline/3`, but returns the result directly instead of wrapping it\nin an `{:ok, result}` tuple or raises if there's an error.\n\nThis function accepts the same options as `pipeline/3`.","ref":"Redix.html#transaction_pipeline!/3"},{"type":"function","title":"Examples - Redix.transaction_pipeline!/3","doc":"iex> Redix.transaction_pipeline!(conn, [[\"SET\", \"mykey\", \"foo\"], [\"GET\", \"mykey\"]])\n    [\"OK\", \"foo\"]","ref":"Redix.html#transaction_pipeline!/3-examples"},{"type":"type","title":"Redix.command/0","doc":"A command, which is a list of things that can be converted to strings.\n\nFor example, this is a valid command:\n\n    [\"INCR\", :my_key, 1]\n\nWe recommend using strings directly to avoid needless conversions.","ref":"Redix.html#t:command/0"},{"type":"type","title":"Redix.connection/0","doc":"The reference to a Redix connection.","ref":"Redix.html#t:connection/0"},{"type":"type","title":"Redix.password/0","doc":"Passwords that can be passed to the `:password` option (see `start_link/1`).","ref":"Redix.html#t:password/0"},{"type":"type","title":"Redix.sentinel_role/0","doc":"The possible role of a Redis sentinel (see `start_link/1`).","ref":"Redix.html#t:sentinel_role/0"},{"type":"module","title":"Redix.Protocol","doc":"This module provides functions to work with the [Redis binary\nprotocol](http://redis.io/topics/protocol).","ref":"Redix.Protocol.html"},{"type":"function","title":"Redix.Protocol.pack/1","doc":"Packs a list of Elixir terms to a Redis (RESP) array.\n\nThis function returns an iodata (instead of a binary) because the packed\nresult is usually sent to Redis through `:gen_tcp.send/2` or similar. It can\nbe converted to a binary with `IO.iodata_to_binary/1`.\n\nAll elements of `elems` are converted to strings with `to_string/1`, hence\nthis function supports encoding everything that implements `String.Chars`.","ref":"Redix.Protocol.html#pack/1"},{"type":"function","title":"Examples - Redix.Protocol.pack/1","doc":"iex> iodata = Redix.Protocol.pack([\"SET\", \"mykey\", 1])\n    iex> IO.iodata_to_binary(iodata)\n    \"*3\\r\\n$3\\r\\nSET\\r\\n$5\\r\\nmykey\\r\\n$1\\r\\n1\\r\\n\"","ref":"Redix.Protocol.html#pack/1-examples"},{"type":"function","title":"Redix.Protocol.parse/1","doc":"Parses a RESP-encoded value from the given `data`.\n\nReturns `{:ok, value, rest}` if a value is parsed successfully, or a\ncontinuation in the form `{:continuation, fun}` if the data is incomplete.","ref":"Redix.Protocol.html#parse/1"},{"type":"function","title":"Examples - Redix.Protocol.parse/1","doc":"iex> Redix.Protocol.parse(\"+OK\\r\\ncruft\")\n    {:ok, \"OK\", \"cruft\"}\n\n    iex> Redix.Protocol.parse(\"-ERR wrong type\\r\\n\")\n    {:ok, %Redix.Error{message: \"ERR wrong type\"}, \"\"}\n\n    iex> {:continuation, fun} = Redix.Protocol.parse(\"+OK\")\n    iex> fun.(\"\\r\\n\")\n    {:ok, \"OK\", \"\"}","ref":"Redix.Protocol.html#parse/1-examples"},{"type":"function","title":"Redix.Protocol.parse_multi/2","doc":"Parses `n` RESP-encoded values from the given `data`.\n\nEach element is parsed as described in `parse/1`. If an element can't be fully\nparsed or there are less than `n` elements encoded in `data`, then a\ncontinuation in the form of `{:continuation, fun}` is returned. Otherwise,\n`{:ok, values, rest}` is returned. If there's an error in decoding, a\n`Redix.Protocol.ParseError` exception is raised.","ref":"Redix.Protocol.html#parse_multi/2"},{"type":"function","title":"Examples - Redix.Protocol.parse_multi/2","doc":"iex> Redix.Protocol.parse_multi(\"+OK\\r\\n+COOL\\r\\n\", 2)\n    {:ok, [\"OK\", \"COOL\"], \"\"}\n\n    iex> {:continuation, fun} = Redix.Protocol.parse_multi(\"+OK\\r\\n\", 2)\n    iex> fun.(\"+OK\\r\\n\")\n    {:ok, [\"OK\", \"OK\"], \"\"}","ref":"Redix.Protocol.html#parse_multi/2-examples"},{"type":"type","title":"Redix.Protocol.on_parse/1","doc":"The return value of parsing functions in this module.","ref":"Redix.Protocol.html#t:on_parse/1"},{"type":"type","title":"Redix.Protocol.redis_value/0","doc":"Represents a Redis value.","ref":"Redix.Protocol.html#t:redis_value/0"},{"type":"module","title":"Redix.PubSub","doc":"Interface for the Redis pub/sub functionality.\n\nThe rest of this documentation will assume the reader knows how pub/sub works\nin Redis and knows the meaning of the following Redis commands:\n\n  * `SUBSCRIBE` and `UNSUBSCRIBE`\n  * `PSUBSCRIBE` and `PUNSUBSCRIBE`\n  * `PUBLISH`","ref":"Redix.PubSub.html"},{"type":"module","title":"Usage - Redix.PubSub","doc":"Each `Redix.PubSub` process is able to subscribe to/unsubscribe from multiple\nRedis channels/patterns, and is able to handle multiple Elixir processes subscribing\neach to different channels/patterns.\n\nA `Redix.PubSub` process can be started via `Redix.PubSub.start_link/2`; such\na process holds a single TCP (or SSL) connection to the Redis server.\n\n`Redix.PubSub` has a message-oriented API. Subscribe operations are synchronous and return\na reference that can then be used to match on all messages sent by the `Redix.PubSub` process.\n\nWhen `Redix.PubSub` registers a subscriptions, the subscriber process will receive a\nconfirmation message:\n\n    {:ok, pubsub} = Redix.PubSub.start_link()\n    {:ok, ref} = Redix.PubSub.subscribe(pubsub, \"my_channel\", self())\n\n    receive do message -> message end\n    #=> {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: \"my_channel\"}}\n\nWhen the `:subscribed` message is received, it's guaranteed that the `Redix.PubSub` process has\nsubscribed to the given channel. This means that after a subscription, messages published to\na channel are delivered to all Elixir processes subscribed to that channel via `Redix.PubSub`:\n\n    # Someone publishes \"hello\" on \"my_channel\"\n    receive do message -> message end\n    #=> {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: \"my_channel\", payload: \"hello\"}}\n\nIt's advised to wait for the subscription confirmation for a channel before doing any\nother operation involving that channel.\n\nNote that unsubscription confirmations are delivered right away even if the `Redix.PubSub`\nprocess is still subscribed to the given channel: this is by design, as once a process\nis unsubscribed from a channel it won't receive messages anyways, even if the `Redix.PubSub`\nprocess still receives them.\n\nMessages are also delivered as a confirmation of an unsubscription as well as when the\n`Redix.PubSub` connection goes down. See the \"Messages\" section below.","ref":"Redix.PubSub.html#module-usage"},{"type":"module","title":"Messages - Redix.PubSub","doc":"Most of the communication with a PubSub connection is done via (Elixir) messages: the\nsubscribers of these messages will be the processes specified at subscription time (in\n`subscribe/3` or `psubscribe/3`). All `Redix.PubSub` messages have the same form: they're a\nfive-element tuple that looks like this:\n\n    {:redix_pubsub, pubsub_pid, subscription_ref, message_type, message_properties}\n\nwhere:\n\n  * `pubsub_pid` is the pid of the `Redix.PubSub` process that sent this message.\n\n  * `subscription_ref` is the reference returned by `subscribe/3` or `psubscribe/3`.\n\n  * `message_type` is the type of this message, such as `:subscribed` for subscription\n    confirmations, `:message` for pub/sub messages, and so on.\n\n  * `message_properties` is a map of data related to that that varies based on `message_type`.\n\nGiven this format, it's easy to match on all Redix pub/sub messages for a subscription\nas `{:redix_pubsub, _, ^subscription_ref, _, _}`.\n\n#","ref":"Redix.PubSub.html#module-messages"},{"type":"module","title":"List of possible message types and properties - Redix.PubSub","doc":"The following is a comprehensive list of possible message types alongside the properties\nthat each can have.\n\n  * `:subscribe` - sent as confirmation of subscription to a channel (via `subscribe/3` or\n    after a disconnection and reconnection). One `:subscribe` message is received for every\n    channel a process subscribed to. `:subscribe` messages have the following properties:\n\n      * `:channel` - the channel the process has been subscribed to.\n\n  * `:psubscribe` - sent as confirmation of subscription to a pattern (via `psubscribe/3` or\n    after a disconnection and reconnection). One `:psubscribe` message is received for every\n    pattern a process subscribed to. `:psubscribe` messages have the following properties:\n\n      * `:pattern` - the pattern the process has been subscribed to.\n\n  * `:unsubscribe` - sent as confirmation of unsubscription from a channel (via\n    `unsubscribe/3`). `:unsubscribe` messages are received for every channel a\n    process unsubscribes from. `:unsubscribe` messages havethe following properties:\n\n      * `:channel` - the channel the process has unsubscribed from.\n\n  * `:punsubscribe` - sent as confirmation of unsubscription from a pattern (via\n    `unsubscribe/3`). `:unsubscribe` messages are received for every pattern a\n    process unsubscribes from. `:unsubscribe` messages havethe following properties:\n\n      * `:pattern` - the pattern the process has unsubscribed from.\n\n  * `:message` - sent to subscribers to a given channel when a message is published on\n    that channel. `:message` messages have the following properties:\n\n      * `:channel` - the channel the message was published on\n      * `:payload` - the contents of the message\n\n  * `:pmessage` - sent to subscribers to a given pattern when a message is published on\n    a channel that matches that pattern. `:pmessage` messages have the following properties:\n\n      * `:channel` - the channel the message was published on\n      * `:pattern` - the original pattern that matched the channel\n      * `:payload` - the contents of the message\n\n  * `:disconnected` messages - sent to all subscribers to all channels/patterns when the\n    connection to Redis is interrupted. `:disconnected` messages have the following properties:\n\n      * `:error` - the reason for the disconnection, a `Redix.ConnectionError`\n        exception struct (that can be raised or turned into a message through\n        `Exception.message/1`).","ref":"Redix.PubSub.html#module-list-of-possible-message-types-and-properties"},{"type":"module","title":"Reconnections - Redix.PubSub","doc":"`Redix.PubSub` tries to be resilient to failures: when the connection with\nRedis is interrupted (for whatever reason), it will try to reconnect to the\nRedis server. When a disconnection happens, `Redix.PubSub` will notify all\nclients subscribed to all channels with a `{:redix_pubsub, pid, subscription_ref, :disconnected,\n_}` message (more on the format of messages above). When the connection goes\nback up, `Redix.PubSub` takes care of actually re-subscribing to the\nappropriate channels on the Redis server and subscribers are notified with a\n`{:redix_pubsub, pid, subscription_ref, :subscribed | :psubscribed, _}` message, the same as\nwhen a client subscribes to a channel/pattern.\n\nNote that if `exit_on_disconnection: true` is passed to\n`Redix.PubSub.start_link/2`, the `Redix.PubSub` process will exit and not send\nany `:disconnected` messages to subscribed clients.","ref":"Redix.PubSub.html#module-reconnections"},{"type":"module","title":"Sentinel support - Redix.PubSub","doc":"Works exactly the same as for normal `Redix` connections. See the documentation for `Redix`\nfor more information.","ref":"Redix.PubSub.html#module-sentinel-support"},{"type":"module","title":"Examples - Redix.PubSub","doc":"This is an example of a workflow using the PubSub functionality; it uses\n[Redix](https://github.com/whatyouhide/redix) as a Redis client for publishing\nmessages.\n\n    {:ok, pubsub} = Redix.PubSub.start_link()\n    {:ok, client} = Redix.start_link()\n\n    Redix.PubSub.subscribe(pubsub, \"my_channel\", self())\n    #=> {:ok, ref}\n\n    # We wait for the subscription confirmation\n    receive do\n      {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: \"my_channel\"}} -> :ok\n    end\n\n    Redix.command!(client, ~w(PUBLISH my_channel hello)\n\n    receive do\n      {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: \"my_channel\"} = properties} ->\n        properties.payload\n    end\n    #=> \"hello\"\n\n    Redix.PubSub.unsubscribe(pubsub, \"foo\", self())\n    #=> :ok\n\n    # We wait for the unsubscription confirmation\n    receive do\n      {:redix_pubsub, ^pubsub, ^ref, :unsubscribed, _} -> :ok\n    end","ref":"Redix.PubSub.html#module-examples"},{"type":"function","title":"Redix.PubSub.get_client_id/1","doc":"Gets the Redis `CLIENT ID` associated with a connection.\n\nThis is useful for implementing [**client-side\ncaching**](https://redis.io/docs/manual/client-side-caching/), where you can\nsubscribe your pub/sub connection to changes on keys.\n\nIf the pub/sub connection is currently disconnected, this function returns\n`{:error, error}`.\n\nThis function requires the `Redix.PubSub` connection to have been started\nwith the `fetch_client_id_on_connect: true` option. This requires\nRedis 5.0.0 or later, since that's where the\n[`CLIENT ID` command](https://redis.io/commands/client-id/)\nwas introduced.","ref":"Redix.PubSub.html#get_client_id/1"},{"type":"function","title":"Examples - Redix.PubSub.get_client_id/1","doc":"iex> Redix.PubSub.get_client_id(conn)\n    {:ok, 123}\n\nIf the connection is not currently connected:\n\n    iex> Redix.PubSub.get_client_id(conn)\n    {:error, %Redix.ConnectionError{reason: :disconnected}\n\nIf the connection was not storing the client ID:\n\n    iex> Redix.PubSub.get_client_id(conn)\n    {:error, %Redix.ConnectionError{reason: :client_id_not_stored}","ref":"Redix.PubSub.html#get_client_id/1-examples"},{"type":"function","title":"Redix.PubSub.psubscribe/3","doc":"Subscribes `subscriber` to the given pattern or list of patterns.\n\nWorks like `subscribe/3` but subscribing `subscriber` to a pattern (or list of\npatterns) instead of regular channels.\n\nUpon successful subscription to each of the `patterns`, a message will be sent\nto `subscriber` with the following form:\n\n    {:redix_pubsub, pid, ^subscription_ref, :psubscribed, %{pattern: pattern}}\n\nSee the documentation for `Redix.PubSub` for more information about the format\nof messages.","ref":"Redix.PubSub.html#psubscribe/3"},{"type":"function","title":"Examples - Redix.PubSub.psubscribe/3","doc":"iex> Redix.psubscribe(conn, \"ba*\", self())\n    :ok\n    iex> flush()\n    {:redix_pubsub, ^conn, ^subscription_ref, :psubscribe, %{pattern: \"ba*\"}}\n    :ok","ref":"Redix.PubSub.html#psubscribe/3-examples"},{"type":"function","title":"Redix.PubSub.punsubscribe/3","doc":"Unsubscribes `subscriber` from the given pattern or list of patterns.\n\nThis function basically \"undoes\" what `psubscribe/3` does: it unsubscribes\n`subscriber` from the given pattern or list of patterns.\n\nUpon successful unsubscription from each of the `patterns`, a message will be\nsent to `subscriber` with the following form:\n\n    {:redix_pubsub, pid, ^subscription_ref, :punsubscribed, %{pattern: pattern}}\n\nSee the documentation for `Redix.PubSub` for more information about the format\nof messages.","ref":"Redix.PubSub.html#punsubscribe/3"},{"type":"function","title":"Examples - Redix.PubSub.punsubscribe/3","doc":"iex> Redix.punsubscribe(conn, \"foo_*\", self())\n    :ok\n    iex> flush()\n    {:redix_pubsub, ^conn, ^subscription_ref, :punsubscribed, %{pattern: \"foo_*\"}}\n    :ok","ref":"Redix.PubSub.html#punsubscribe/3-examples"},{"type":"function","title":"Redix.PubSub.start_link/1","doc":"Starts a pub/sub connection to Redis.\n\nThis function returns `{:ok, pid}` if the PubSub process is started successfully.\n\nThe actual TCP/SSL connection to the Redis server may happen either synchronously,\nbefore `start_link/2` returns, or asynchronously: this behaviour is decided by\nthe `:sync_connect` option (see below).\n\nThis function accepts one argument, either a Redis URI as a string or a list of options.","ref":"Redix.PubSub.html#start_link/1"},{"type":"function","title":"Redis URI - Redix.PubSub.start_link/1","doc":"In case `uri_or_opts` is a Redis URI, it must be in the form:\n\n    redis://[:password@]host[:port][/db]\n\nHere are some examples of valid URIs:\n\n    redis://localhost\n    redis://:secret@localhost:6397\n    redis://username:secret@localhost:6397\n    redis://example.com:6380/1\n\nThe only mandatory thing when using URIs is the host. All other elements are optional\nand their default value can be found in the \"Options\" section below.\n\nIn earlier versions of Redix, the username in the URI was ignored. Redis 6 introduced [ACL\nsupport](https://redis.io/topics/acl). Now, Redix supports usernames as well.","ref":"Redix.PubSub.html#start_link/1-redis-uri"},{"type":"function","title":"Options - Redix.PubSub.start_link/1","doc":"The following options can be used to specify the connection:\n\n* `:host` (`t:String.t/0`) - the host where the Redis server is running. If you are using a Redis URI, you cannot\n  use this option. Defaults to `\"localhost`\".\n\n* `:port` (`t:non_neg_integer/0`) - the port on which the Redis server is running. If you are using a Redis URI, you cannot\n  use this option. Defaults to `6379`.\n\n* `:database` (`t:String.t/0` or `t:non_neg_integer/0`) - the database to connect to. Defaults to `nil`, meaning Redix doesn't connect to a\n  specific database (the default in this case is database `0`). When this option is provided,\n  all Redix does is issue a `SELECT` command to Redis in order to select the given database.\n\n* `:username` - the username to connect to Redis. Defaults to `nil`, meaning no username is used.\n  Redis supports usernames only since Redis 6 (see the [ACL\n  documentation](https://redis.io/topics/acl)). If a username is provided (either via\n  options or via URIs) and the Redis version used doesn't support ACL, then Redix falls\n  back to using just the password and emits a warning. In future Redix versions, Redix\n  will raise if a username is passed and the Redis version used doesn't support ACL.\n\n* `:password` (`t:Redix.password/0`) - the password used to connect to Redis. Defaults to\n  `nil`, meaning no password is used. When this option is provided, all Redix\n  does is issue an `AUTH` command to Redis in order to authenticate. MFAs are also\n  supported in the form of `{module, function, arguments}`. This can be used\n  to fetch the password dynamically on every reconnection but most importantly to\n  hide the password from crash reports in case the Redix connection crashes for\n  any reason. For example, you can set this option to:\n  `{System, :fetch_env!, [\"REDIX_PASSWORD\"]}`.\n\n* `:timeout` (`t:timeout/0`) - connection timeout (in milliseconds) directly passed to the network layer. The default value is `5000`.\n\n* `:sync_connect` (`t:boolean/0`) - decides whether Redix should initiate the network connection to the Redis server *before*\n  or *after* returning from `start_link/1`. This option also changes some reconnection\n  semantics; read the \"Reconnections\" page in the documentation for more information. The default value is `false`.\n\n* `:exit_on_disconnection` (`t:boolean/0`) - if `true`, the Redix server will exit if it fails to connect or disconnects from Redis.\n  Note that setting this option to `true` means that the `:backoff_initial` and\n  `:backoff_max` options will be ignored. The default value is `false`.\n\n* `:backoff_initial` (`t:non_neg_integer/0`) - the initial backoff time (in milliseconds), which is the time that the Redix process\n  will wait before attempting to reconnect to Redis after a disconnection or failed first\n  connection. See the \"Reconnections\" page in the docs for more information. The default value is `500`.\n\n* `:backoff_max` (`t:timeout/0`) - the maximum length (in milliseconds) of the time interval used between reconnection\n  attempts. See the \"Reconnections\" page in the docs for more information. The default value is `30000`.\n\n* `:ssl` (`t:boolean/0`) - if `true`, connect through SSL, otherwise through TCP. The `:socket_opts` option applies\n  to both SSL and TCP, so it can be used for things like certificates. See `:ssl.connect/4`. The default value is `false`.\n\n* `:name` (`t:term/0`) - Redix is bound to the same registration rules as a `GenServer`. See the `GenServer`\n  documentation for more information.\n\n* `:socket_opts` (list of `t:term/0`) - specifies a list of options that are passed to the network layer when connecting to\n  the Redis server. Some socket options (like `:active` or `:binary`) will be\n  overridden by Redix so that it functions properly.\n\n  If `ssl: true`, then these are added to the default: `[verify: :verify_peer, depth: 3]`.\n  If the `CAStore` dependency is available, the `:cacertfile` option is added\n  to the SSL options by default as well.\n\n  The default value is `[]`.\n\n* `:hibernate_after` (`t:non_neg_integer/0`) - if present, the Redix connection process awaits any message for the given number\n  of milliseconds and if no message is received, the process goes into hibernation\n  automatically (by calling `:proc_lib.hibernate/3`). See `t::gen_statem.start_opt/0`.\n  Not present by default.\n\n* `:spawn_opt` (`t:keyword/0`) - if present, its value is passed as options to the Redix connection process as in\n  `Process.spawn/4`. See `t::gen_statem.start_opt/0`. Not present by default.\n\n* `:debug` (`t:keyword/0`) - if present, the corresponding function in the\n  [`:sys` module](http://www.erlang.org/doc/man/sys.html) is invoked.\n\n* `:fetch_client_id_on_connect` (`t:boolean/0`) - if `true`, Redix will fetch the client ID after connecting to Redis and before\n  subscribing to any topic. You can then read the client ID of the pub/sub connection\n  with `get_client_id/1`. This option uses the `CLIENT ID` command under the hood,\n  which is available since Redis 5.0.0. *This option is available since v1.4.1*. The default value is `false`.\n\n* `:sentinel` (`t:keyword/0`) - options to use Redis Sentinel. If this option is present, you cannot use the `:host` and\n  `:port` options. See the [*Sentinel Options* section below](#start_link/1-sentinel-options).\n\n#","ref":"Redix.PubSub.html#start_link/1-options"},{"type":"function","title":"Sentinel Options - Redix.PubSub.start_link/1","doc":"* `:sentinels` (list of `t:String.t/0` or `t:keyword/0`) - Required. a list of sentinel addresses. Each element in this list is the address\n  of a sentinel to be contacted in order to obtain the address of a primary. The address of\n  a sentinel can be passed as a Redis URI (see the \"Using a Redis URI\" section) or\n  a keyword list with `:host`, `:port`, `:password` options (same as when connecting to a\n  Redis instance directly). Note that the password can either be passed in the sentinel\n  address or globally — see the `:password` option below.\n\n* `:group` (`t:String.t/0`) - Required. the name of the group that identifies the primary in the sentinel configuration.\n\n* `:role` (`t:Redix.sentinel_role/0`) - if `:primary`, the connection will be established\n  with the primary for the given group. If `:replica`, Redix will ask the sentinel for all\n  the available replicas for the given group and try to connect to one of them\n  **at random**. The default value is `:primary`.\n\n* `:socket_opts` (`t:keyword/0`) - socket options for connecting to each sentinel. Same as the `:socket_opts` option\n  described above. The default value is `[]`.\n\n* `:timeout` (`t:timeout/0`) - the timeout (in milliseconds or `:infinity`) that will be used to\n  interact with the sentinels. This timeout will be used as the timeout when connecting to\n  each sentinel and when asking sentinels for a primary. The Redis documentation suggests\n  to keep this timeout short so that connection to Redis can happen quickly. The default value is `500`.\n\n* `:ssl` (`t:boolean/0`) - whether to use SSL to connect to each sentinel. The default value is `false`.\n\n* `:password` (`t:Redix.password/0`) - if you don't want to specify a password for each sentinel you\n  list, you can use this option to specify a password that will be used to authenticate\n  on sentinels if they don't specify a password. This option is recommended over passing\n  a password for each sentinel because in the future we might do sentinel auto-discovery,\n  which means authentication can only be done through a global password that works for all\n  sentinels.","ref":"Redix.PubSub.html#start_link/1-sentinel-options"},{"type":"function","title":"Examples - Redix.PubSub.start_link/1","doc":"iex> Redix.PubSub.start_link()\n    {:ok, #PID<...>}\n\n    iex> Redix.PubSub.start_link(host: \"example.com\", port: 9999, password: \"secret\")\n    {:ok, #PID<...>}\n\n    iex> Redix.PubSub.start_link([database: 3], [name: :redix_3])\n    {:ok, #PID<...>}","ref":"Redix.PubSub.html#start_link/1-examples"},{"type":"function","title":"Redix.PubSub.start_link/2","doc":"Same as `start_link/1` but using both a Redis URI and a list of options.\n\nIn this case, options specified in `opts` have precedence over values specified by `uri`.\nFor example, if `uri` is `redix://example1.com` but `opts` is `[host: \"example2.com\"]`, then\n`example2.com` will be used as the host when connecting.","ref":"Redix.PubSub.html#start_link/2"},{"type":"function","title":"Redix.PubSub.stop/2","doc":"Stops the given pub/sub process.\n\nThis function is synchronous and blocks until the given pub/sub connection\nfrees all its resources and disconnects from the Redis server. `timeout` can\nbe passed to limit the amount of time allowed for the connection to exit; if\nit doesn't exit in the given interval, this call exits.","ref":"Redix.PubSub.html#stop/2"},{"type":"function","title":"Examples - Redix.PubSub.stop/2","doc":"iex> Redix.PubSub.stop(conn)\n    :ok","ref":"Redix.PubSub.html#stop/2-examples"},{"type":"function","title":"Redix.PubSub.subscribe/3","doc":"Subscribes `subscriber` to the given channel or list of channels.\n\nSubscribes `subscriber` (which can be anything that can be passed to `send/2`)\nto `channels`, which can be a single channel or a list of channels.\n\nFor each of the channels in `channels` which `subscriber` successfully\nsubscribes to, a message will be sent to `subscriber` with this form:\n\n    {:redix_pubsub, pid, subscription_ref, :subscribed, %{channel: channel}}\n\nSee the documentation for `Redix.PubSub` for more information about the format\nof messages.","ref":"Redix.PubSub.html#subscribe/3"},{"type":"function","title":"Examples - Redix.PubSub.subscribe/3","doc":"iex> Redix.PubSub.subscribe(conn, [\"foo\", \"bar\"], self())\n    {:ok, subscription_ref}\n    iex> flush()\n    {:redix_pubsub, ^conn, ^subscription_ref, :subscribed, %{channel: \"foo\"}}\n    {:redix_pubsub, ^conn, ^subscription_ref, :subscribed, %{channel: \"bar\"}}\n    :ok","ref":"Redix.PubSub.html#subscribe/3-examples"},{"type":"function","title":"Redix.PubSub.unsubscribe/3","doc":"Unsubscribes `subscriber` from the given channel or list of channels.\n\nThis function basically \"undoes\" what `subscribe/3` does: it unsubscribes\n`subscriber` from the given channel or list of channels.\n\nUpon successful unsubscription from each of the `channels`, a message will be\nsent to `subscriber` with the following form:\n\n    {:redix_pubsub, pid, ^subscription_ref, :unsubscribed, %{channel: channel}}\n\nSee the documentation for `Redix.PubSub` for more information about the format\nof messages.","ref":"Redix.PubSub.html#unsubscribe/3"},{"type":"function","title":"Examples - Redix.PubSub.unsubscribe/3","doc":"iex> Redix.unsubscribe(conn, [\"foo\", \"bar\"], self())\n    :ok\n    iex> flush()\n    {:redix_pubsub, ^conn, ^subscription_ref, :unsubscribed, %{channel: \"foo\"}}\n    {:redix_pubsub, ^conn, ^subscription_ref, :unsubscribed, %{channel: \"bar\"}}\n    :ok","ref":"Redix.PubSub.html#unsubscribe/3-examples"},{"type":"type","title":"Redix.PubSub.connection/0","doc":"","ref":"Redix.PubSub.html#t:connection/0"},{"type":"type","title":"Redix.PubSub.subscriber/0","doc":"","ref":"Redix.PubSub.html#t:subscriber/0"},{"type":"module","title":"Redix.Telemetry","doc":"Telemetry integration for event tracing, metrics, and logging.\n\nRedix connections (both `Redix` and `Redix.PubSub`) execute the\nfollowing Telemetry events:\n\n  * `[:redix, :connection]` - executed when a Redix connection establishes the\n    connection to Redis. There are no measurements associated with this event.\n    Metadata are:\n\n    * `:connection` - the PID of the Redix connection that emitted the event.\n    * `:connection_name` - the name (passed to the `:name` option when the\n    connection is started) of the Redix connection that emitted the event.\n    `nil` if the connection was not registered with a name.\n    * `:address` - the address the connection successfully connected to.\n    * `:reconnection` - a boolean that specifies whether this was a first\n      connection to Redis or a reconnection after a disconnection. This can\n      be useful for more granular logging.\n\n  * `[:redix, :disconnection]` - executed when the connection is lost\n    with the Redis server. There are no measurements associated with\n    this event. Metadata are:\n\n    * `:connection` - the PID of the Redix connection that emitted the event.\n    * `:connection_name` - the name (passed to the `:name` option when the\n    * `:address` - the address the connection was connected to.\n      connection is started) of the Redix connection that emitted the event.\n      `nil` if the connection was not registered with a name.\n    * `:reason` - the disconnection reason as a `Redix.ConnectionError` struct.\n\n  * `[:redix, :failed_connection]` - executed when Redix can't connect to\n    the specified Redis server, either when starting up the connection or\n    after a disconnection. There are no measurements associated with this event.\n    Metadata are:\n\n    * `:connection` - the PID of the Redix connection that emitted the event.\n    * `:connection_name` - the name (passed to the `:name` option when the\n      connection is started) of the Redix connection that emitted the event.\n      `nil` if the connection was not registered with a name.\n    * `:address` or `:sentinel_address` - the address the connection was trying\n    to connect to (either a Redis server or a Redis Sentinel instance).\n    * `:reason` - the disconnection reason as a `Redix.ConnectionError` struct.\n\n`Redix` connections execute the following Telemetry events when commands or\npipelines of any kind are executed.\n\n  * `[:redix, :pipeline, :start]` - executed right before a pipeline (or command,\n    which is a pipeline with just one command) is sent to the Redis server.\n    Measurements are:\n\n    * `:system_time` (integer) - the system time (in the `:native` time unit)\n      at the time the event is emitted. See `System.system_time/0`.\n\n    Metadata are:\n\n    * `:connection` - the PID of the Redix connection used to send the pipeline.\n    * `:connection_name` - the name of the Redix connection used to sent the pipeline.\n      This is `nil` if the connection was not registered with a name or if the\n      pipeline function was called with a PID directly (for example, if you did\n      `Process.whereis/1` manually).\n    * `:commands` - the commands sent to the server. This is always a list of\n      commands, so even if you do `Redix.command(conn, [\"PING\"])` then the\n      list of commands will be `[[\"PING\"]]`.\n    * `:extra_metadata` - any term set by users via the `:telemetry_metadata` option\n      in `Redix.pipeline/3` and other functions.\n\n  * `[:redix, :pipeline, :stop]` - executed a response to a pipeline returns\n    from the Redis server, regardless of whether it's an error response or a\n    successful response. Measurements are:\n\n    * `:duration` - the duration (in the `:native` time unit, see `t:System.time_unit/0`)\n      of back-and-forth between client and server.\n\n    Metadata are:\n\n    * `:connection` - the PID of the Redix connection used to send the pipeline.\n    * `:connection_name` - the name of the Redix connection used to sent the pipeline.\n      This is `nil` if the connection was not registered with a name or if the\n      pipeline function was called with a PID directly (for example, if you did\n      `Process.whereis/1` manually).\n    * `:commands` - the commands sent to the server. This is always a list of\n      commands, so even if you do `Redix.command(conn, [\"PING\"])` then the\n      list of commands will be `[[\"PING\"]]`.\n    * `:extra_metadata` - any term set by users via the `:telemetry_metadata` option\n      in `Redix.pipeline/3` and other functions.\n\n    If the response is an error, the following metadata will also be present:\n\n    * `:kind` - the atom `:error`.\n    * `:reason` - the error reason (such as a `Redix.ConnectionError` struct).\n\nMore events might be added in the future and that won't be considered a breaking\nchange, so if you're writing a handler for Redix events be sure to ignore events\nthat are not known. All future Redix events will start with the `:redix` atom,\nlike the ones above.\n\nA default handler that logs these events appropriately is provided, see\n`attach_default_handler/0`. Otherwise, you can write your own handler to\ninstrument or log events, see the [Telemetry page](telemetry.html) in the docs.","ref":"Redix.Telemetry.html"},{"type":"function","title":"Redix.Telemetry.attach_default_handler/0","doc":"Attaches the default Redix-provided Telemetry handler.\n\nThis function attaches a default Redix-provided handler that logs\n(using Elixir's `Logger`) the following events:\n\n  * `[:redix, :disconnection]` - logged at the `:error` level\n  * `[:redix, :failed_connection]` - logged at the `:error` level\n  * `[:redix, :connection]` - logged at the `:info` level if it's a\n    reconnection, not logged if it's the first connection.\n\nSee the module documentation for more information. If you want to\nattach your own handler, look at the [Telemetry page](telemetry.html)\nin the documentation.","ref":"Redix.Telemetry.html#attach_default_handler/0"},{"type":"function","title":"Examples - Redix.Telemetry.attach_default_handler/0","doc":":ok = Redix.Telemetry.attach_default_handler()","ref":"Redix.Telemetry.html#attach_default_handler/0-examples"},{"type":"module","title":"Redix.URI","doc":"This module provides functions to work with a Redis URI.\n\nThis is generally intended for library developers using Redix under the hood.","ref":"Redix.URI.html"},{"type":"function","title":"Redix.URI.to_start_options/1","doc":"Returns start options from a Redis URI.\n\nA **Redis URI** looks like this:\n\n    redis://[username:password@]host[:port][/db]\n\n> ##","ref":"Redix.URI.html#to_start_options/1"},{"type":"function","title":"Valkey {: .tip} - Redix.URI.to_start_options/1","doc":">\n> URIs also work with [Valkey](https://valkey.io/), a Redis-compatible in-memory key-value\n> store. Use `valkey://` as the scheme instead of `redis://`.","ref":"Redix.URI.html#to_start_options/1-valkey-tip"},{"type":"function","title":"Examples - Redix.URI.to_start_options/1","doc":"iex> Redix.URI.to_start_options(\"redis://example.com\")\n    [host: \"example.com\"]\n\n    iex> Redix.URI.to_start_options(\"rediss://username:password@example.com:5000/3\")\n    [ssl: true, database: 3, password: \"password\", username: \"username\", port: 5000, host: \"example.com\"]","ref":"Redix.URI.html#to_start_options/1-examples"},{"type":"exception","title":"Redix.ConnectionError","doc":"Error in the connection to Redis.\n\nThis exception represents errors in the connection to Redis: for example,\nrequest timeouts, disconnections, and similar.","ref":"Redix.ConnectionError.html"},{"type":"exception","title":"Exception fields - Redix.ConnectionError","doc":"See `t:t/0`.","ref":"Redix.ConnectionError.html#module-exception-fields"},{"type":"exception","title":"Error reasons - Redix.ConnectionError","doc":"The `:reason` field can assume a few Redix-specific values:\n\n  * `:closed`: when the connection to Redis is closed (and Redix is\n    reconnecting) and the user attempts to talk to Redis\n\n  * `:disconnected`: when the connection drops while a request to Redis is in\n    flight.\n\n  * `:timeout`: when Redis doesn't reply to the request in time.","ref":"Redix.ConnectionError.html#module-error-reasons"},{"type":"type","title":"Redix.ConnectionError.t/0","doc":"The type for this exception struct.\n\nThis exception has the following public fields:\n\n  * `:reason` - the error reason. It can be one of the Redix-specific\n    reasons described in the \"Error reasons\" section below, or any error\n    reason returned by functions in the `:gen_tcp` module (see the\n    [`:inet.posix/0`](http://www.erlang.org/doc/man/inet.html#type-posix) type) or\n    `:ssl` module.","ref":"Redix.ConnectionError.html#t:t/0"},{"type":"exception","title":"Redix.Error","doc":"Error returned by Redis.\n\nThis exception represents semantic errors returned by Redis: for example,\nnon-existing commands or operations on keys with the wrong type (`INCR\nnot_an_integer`).","ref":"Redix.Error.html"},{"type":"type","title":"Redix.Error.t/0","doc":"The type for this exception struct.","ref":"Redix.Error.html#t:t/0"},{"type":"exception","title":"Redix.Protocol.ParseError","doc":"Error in parsing data according to the\n[RESP](http://redis.io/topics/protocol) protocol.","ref":"Redix.Protocol.ParseError.html"},{"type":"extras","title":"Redix","doc":"# Redix\n\n[![hex.pm badge](https://img.shields.io/badge/Package%20on%20hex.pm-informational)](https://hex.pm/packages/redix)\n[![Documentation badge](https://img.shields.io/badge/Documentation-ff69b4)][docs]\n[![CI](https://github.com/whatyouhide/redix/actions/workflows/main.yml/badge.svg)](https://github.com/whatyouhide/redix/actions/workflows/main.yml)\n[![Coverage Status](https://coveralls.io/repos/github/whatyouhide/redix/badge.svg?branch=main)](https://coveralls.io/github/whatyouhide/redix?branch=main)\n\n> Fast, pipelined, resilient Redis client for Elixir.\n\n![dalle](https://user-images.githubusercontent.com/3890250/208236010-a89acc2a-ac79-44ca-9413-c1fa7c664ffe.jpg)\n\nRedix is a [Redis][redis] and [Valkey][valkey] client written in pure Elixir with focus on speed, correctness, and resiliency (that is, being able to automatically reconnect to Redis in case of network errors).\n\nThis README refers to the main branch of Redix, not the latest released version on Hex. Make sure to check [the documentation][docs] for the version you're using.","ref":"readme.html"},{"type":"extras","title":"Features - Redix","doc":"* Idiomatic interface for sending commands to Redis\n  * Pipelining\n  * Resiliency (automatic reconnections)\n  * Pub/Sub\n  * SSL\n  * Redis Sentinel","ref":"readme.html#features"},{"type":"extras","title":"Installation - Redix","doc":"Add the `:redix` dependency to your `mix.exs` file. If you plan on connecting to a Redis server [over SSL][docs-ssl] you may want to add the optional [`:castore`][castore] dependency as well:\n\n```elixir\ndefp deps do\n  [\n    {:redix, \"~> 1.1\"},\n    {:castore, \">= 0.0.0\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` in your shell to fetch the new dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Redix","doc":"Redix is simple: it doesn't wrap Redis commands with Elixir functions. It only provides functions to send any Redis command to the Redis server. A Redis *command* is expressed as a list of strings making up the command and its arguments.\n\nConnections are started via `start_link/0,1,2`:\n\n```elixir\n{:ok, conn} = Redix.start_link(host: \"example.com\", port: 5000)\n{:ok, conn} = Redix.start_link(\"redis://localhost:6379/3\", name: :redix)\n```\n\nCommands can be sent using `Redix.command/2,3`:\n\n```elixir\nRedix.command(conn, [\"SET\", \"mykey\", \"foo\"])\n#=> {:ok, \"OK\"}\nRedix.command(conn, [\"GET\", \"mykey\"])\n#=> {:ok, \"foo\"}\n```\n\nPipelines are just lists of commands sent all at once to Redis for which Redis replies with a list of responses. They can be used in Redix via `Redix.pipeline/2,3`:\n\n```elixir\nRedix.pipeline(conn, [[\"INCR\", \"foo\"], [\"INCR\", \"foo\"], [\"INCRBY\", \"foo\", \"2\"]])\n#=> {:ok, [1, 2, 4]}\n```\n\n`Redix.command/2,3` and `Redix.pipeline/2,3` always return `{:ok, result}` or `{:error, reason}`. If you want to access the result directly and raise in case there's an error, bang! variants are provided:\n\n```elixir\nRedix.command!(conn, [\"PING\"])\n#=> \"PONG\"\n\nRedix.pipeline!(conn, [[\"SET\", \"mykey\", \"foo\"], [\"GET\", \"mykey\"]])\n#=> [\"OK\", \"foo\"]\n```\n\n##","ref":"readme.html#usage"},{"type":"extras","title":"Resiliency - Redix","doc":"Redix is resilient against network errors. For example, if the connection to Redis drops, Redix will automatically try to reconnect periodically at a given \"backoff\" interval. Look at the documentation for the `Redix` module and at the [\"Reconnections\" page][docs-reconnections] in the documentation for more information on the available options and on the exact reconnection behaviour.\n\n##","ref":"readme.html#resiliency"},{"type":"extras","title":"Redis Sentinel - Redix","doc":"Redix supports [Redis Sentinel][redis-sentinel] out of the box. You can specify a list of sentinels to connect to when starting a `Redix` (or `Redix.PubSub`) connection. Every time that connection will need to connect to a Redis server (the first time or after a disconnection), it will try to connect to one of the sentinels in order to ask that sentinel for the current primary or a replica.\n\n```elixir\nsentinels = [\"redis://sent1.example.com:26379\", \"redis://sent2.example.com:26379\"]\n{:ok, primary} = Redix.start_link(sentinel: [sentinels: sentinels, group: \"main\"])\n```\n\n###","ref":"readme.html#redis-sentinel"},{"type":"extras","title":"Terminology - Redix","doc":"Redix doesn't support the use of the terms \"master\" and \"slave\" that are usually used with Redis Sentinel. I don't think those are good terms to use, period. Instead, Redix uses the terms \"primary\" and \"replica\". If you're interested in the discussions around this, [this][redis-terminology-issue] issue in the Redis repository might be interesting to you.\n\n##","ref":"readme.html#terminology"},{"type":"extras","title":"Pub/Sub - Redix","doc":"A `Redix.PubSub` process can be started via `Redix.PubSub.start_link/2`:\n\n```elixir\n{:ok, pubsub} = Redix.PubSub.start_link()\n```\n\nMost communication with the `Redix.PubSub` process happens via Elixir messages (that simulate a Pub/Sub interaction with the pub/sub server).\n\n```elixir\n{:ok, pubsub} = Redix.PubSub.start_link()\n\nRedix.PubSub.subscribe(pubsub, \"my_channel\", self())\n#=> {:ok, ref}\n```\n\nConfirmation of subscriptions is delivered as an Elixir message:\n\n```elixir\nreceive do\n  {:redix_pubsub, ^pubsub, ^ref, :subscribed, %{channel: \"my_channel\"}} -> :ok\nend\n```\n\nIf someone publishes a message on a channel we're subscribed to:\n\n```elixir\nreceive do\n  {:redix_pubsub, ^pubsub, ^ref, :message, %{channel: \"my_channel\", payload: \"hello\"}} ->\n    IO.puts(\"Received a message!\")\nend\n```","ref":"readme.html#pub-sub"},{"type":"extras","title":"Using Redix in the Real World™ - Redix","doc":"Redix is low-level, but it's still built to handle most things thrown at it. For many applications, you can avoid pooling with little to no impact on performance. Read the [\"Real world usage\" page][docs-real-world-usage] in the documentation for more information on this and pooling strategies that work better with Redix.","ref":"readme.html#using-redix-in-the-real-world"},{"type":"extras","title":"Contributing - Redix","doc":"To run the Redix test suite you will have to have Redis running locally. Redix requires a somewhat complex setup for running tests (because it needs a few instances running, for pub/sub and sentinel). For this reason, in this repository you'll find a `docker-compose.yml` file so that you can use [Docker][docker] and [docker-compose][] to spin up all the necessary Redis instances with just one command. Make sure you have Docker installed and then just run:\n\n```bash\ndocker-compose up\n```\n\nNow, you're ready to run tests with the `$ mix test` command.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Redix","doc":"Redix is released under the MIT license. See the [license file](LICENSE.txt).\n\n[docs]: http://hexdocs.pm/redix\n[redis]: http://redis.io\n[redis-sentinel]: https://redis.io/topics/sentinel\n[castore]: https://github.com/ericmj/castore\n[docs-ssl]: https://hexdocs.pm/redix/Redix.html#module-ssl\n[docs-reconnections]: http://hexdocs.pm/redix/reconnections.html\n[docs-real-world-usage]: http://hexdocs.pm/redix/real-world-usage.html\n[docker]: https://www.docker.com\n[docker-compose]: https://docs.docker.com/compose/\n[redis-terminology-issue]: https://github.com/antirez/redis/issues/5335\n[valkey]: https://valkey.io/","ref":"readme.html#license"},{"type":"extras","title":"Reconnections","doc":"# Reconnections\n\nRedix tries to be as resilient as possible. When the connection to Redis drops for some reason, a Redix process will try to reconnect to the Redis server.\n\nIf there are pending requests to Redix when a disconnection happens, the `Redix` functions will return `{:error, %Redix.ConnectionError{reason: :disconnected}}` to the caller. The caller is responsible to retry the request if interested.\n\nThe first reconnection attempts happens after a backoff interval decided by the `:backoff_initial` option. If this attempt succeeds, then Redix will start to function normally again. If this attempt fails, then subsequent reconnection attempts are made until one of them succeeds. The backoff interval between these subsequent reconnection attempts is increased exponentially (with a fixed factor of `1.5`). This means that the first attempt will be made after `n` milliseconds, the second one after `n * 1.5` milliseconds, the third one after `n * 1.5 * 1.5` milliseconds, and so on. Since this growth is exponential, it won't take many attempts before this backoff interval becomes large: because of this, `Redix.start_link/2` also accepts a `:backoff_max` option. which specifies the maximum backoff interval that should be used. The `:backoff_max` option can be used to simulate constant backoff after some exponential backoff attempts: for example, by passing `backoff_max: 5_000` and `backoff_initial: 5_000`, attempts will be made regularly every 5 seconds.","ref":"reconnections.html"},{"type":"extras","title":"Synchronous or asynchronous connection - Reconnections","doc":"The `:sync_connect` option passed to `Redix.start_link/2` decides whether Redix should initiate the TCP connection to the Redis server *before* or *after* `Redix.start_link/2` returns. This option also changes the behaviour of Redix when the TCP connection can't be initiated at all.\n\nWhen `:sync_connect` is `false`, then a failed attempt to initially connect to the Redis server is treated exactly as a disconnection: attempts to reconnect are made as described above. This behaviour should be used when Redix is not a vital part of your application: your application should be prepared to handle Redis being down (for example, using the non \"bang\" variants to issue commands to Redis and handling `{:error, _}` tuples).\n\nWhen `:sync_connect` is `true`, then a failed attempt to initiate the connection to Redis will cause the Redix process to fail and exit. This might be what you want if Redis is vital to your application.\n\n#","ref":"reconnections.html#synchronous-or-asynchronous-connection"},{"type":"extras","title":"If Redis is vital to your application - Reconnections","doc":"You should use `sync_connect: true` if Redis is a vital part of your application: for example, if you plan to use a Redix process under your application's supervision tree, placed *before* the parts of your application that depend on it in the tree (so that this way, the application won't be started until a connection to Redis has been established). With `sync_connect: true`, disconnections after the TCP connection has been established will behave exactly as above (with reconnection attempts at given intervals). However, if your application can't function properly without Redix, then you want to use `exit_on_disconnection: true`. With this option, the connection will crash when a disconnection happens. With `:sync_connect` and `:exit_on_disconnection`, you can isolate the part of your application that can't work without Redis under a supervisor and bring that part down when Redix crashes:\n\n```elixir\nisolated_children = [\n  {Redix, sync_connect: true, exit_on_disconnection: true},\n  MyApp.MyGenServer\n]\n\nisolated_supervisor = %{\n  id: MyChildSupervisor,\n  type: :supervisor,\n  start: {Supervisor, :start_link, [isolated_children, [strategy: :rest_for_one]]},\n}\n\nchildren = [\n  MyApp.Child1,\n  isolated_supervisor,\n  MyApp.Child2\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"reconnections.html#if-redis-is-vital-to-your-application"},{"type":"extras","title":"Real-world usage","doc":"# Real-world usage\n\nRedix is a low-level driver, but it's still built to handle most stuff thrown at it.\n\nRedix is built to handle multiple Elixir processes sending commands to Redis through it at the same time. It takes advantage of TCP being a **full-duplex** protocol (bytes are sent in both directions, often at the same time) so that the TCP stream has bytes flowing in both directions (to and from Redis). For example, if two Elixir processes send a `PING` command to Redis via `Redix.command/2`, Redix will send both commands to Redis but will concurrently start listening for the reply to these commands; at a given point, both a `PING` command as well as the `PONG` response to a previous `PING` could be flowing in the TCP stream of the socket that Redix is using.\n\nThere's a few different ways to use Redix and to pool connections for better high-load support.","ref":"real-world-usage.html"},{"type":"extras","title":"Single named Redix instance - Real-world usage","doc":"For many applications, a single global Redix instance is enough. This is true especially for applications where requests to Redis are not mapping one-to-one to things like user requests (that is, a request for each user). A common pattern in these cases is to have a named Redix process started under the supervision tree:\n\n```elixir\nchildren = [\n  {Redix, name: :redix}\n]\n```\n\nOnce Redix is started and registered, you can use it with the given name from anywhere:\n\n```elixir\nRedix.command(:redix, [\"PING\"])\n#=> {:ok, \"PONG\"}\n```\n\nNote that this pattern extends to more than one global (named) Redix: for example, you could have a Redix process for handling big and infrequent requests and another one to handle short and frequent requests.","ref":"real-world-usage.html#single-named-redix-instance"},{"type":"extras","title":"Name-based pool - Real-world usage","doc":"When you want to have a pool of connections, you can start many connections and register them by name. Say you want to have a pool of five Redis connections. You can start these connections in a supervisor under your supervision tree and then create a wrapper module that calls connections from the pool. The wrapper can use any strategy to choose which connection to use, for example a random strategy.\n\n```elixir\ndefmodule MyApp.Redix do\n  @pool_size 5\n\n  def child_spec(_args) do\n    # Specs for the Redix connections.\n    children =\n      for index <- 0..(@pool_size - 1) do\n        Supervisor.child_spec({Redix, name: :\"redix_#{index}\"}, id: {Redix, index})\n      end\n\n    # Spec for the supervisor that will supervise the Redix connections.\n    %{\n      id: RedixSupervisor,\n      type: :supervisor,\n      start: {Supervisor, :start_link, [children, [strategy: :one_for_one]]}\n    }\n  end\n\n  def command(command) do\n    Redix.command(:\"redix_#{random_index()}\", command)\n  end\n\n  defp random_index do\n    Enum.random(0..@pool_size - 1)\n  end\nend\n```\n\nYou can then start the Redix connections and their supervisor in the application's supervision tree:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    MyApp.Redix,\n    # ...other children\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one)\nend\n```\n\nAnd then use the new wrapper in your application:\n\n```elixir\nMyApp.Redix.command([\"PING\"])\n#=> {:ok, \"PONG\"}\n```\n\n#","ref":"real-world-usage.html#name-based-pool"},{"type":"extras","title":"Caveats of the name-based pool - Real-world usage","doc":"The name-based pool works well enough for many use cases but it has a few caveats.\n\nThe first one is that the load of requests to Redis is distributed fairly among the connections in the pool, but is not distributed in a \"smart\" way. For example, you might want to send less requests to connections that are behaving in a worse way, such as slower connections. This avoids bottling up connections that are already slow by sending more requests to them and distributes the load more evenly.\n\nThe other caveat is that you need to think about possible race conditions when using this kind of pool since every time you issue a command you could be using a different connections. If you issue commands from the same process, things will work since the process will block until it receives a reply so we know that Redis received and processed the command before we can issue a new one. However, if you issue commands from different processes, you can't be sure of the order that they get processed by Redis. After all, this is often true when doing things from different processes and is not particularly Redix specific.","ref":"real-world-usage.html#caveats-of-the-name-based-pool"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\nSince version v0.10.0, Redix uses [Telemetry][telemetry] for instrumentation and for having an extensible way of doing logging. Telemetry is a metrics and instrumentation library for Erlang and Elixir applications that is based on publishing events through a common interface and attaching handlers to handle those events. For more information about the library itself, see [its README][telemetry].\n\nBefore version v0.10.0, `Redix.start_link/1` and `Redix.PubSub.start_link/1` supported a `:log` option to control logging. For example, if you wanted to log disconnections at the `:error` level and reconnections and the `:debug` level, you would do:\n\n    Redix.start_link(log: [disconnection: :error, reconnection: :debug])\n\nThe `:log` option is now removed in favour of either using the default Redix event handler or writing your own.\n\nFor information on the Telemetry events that Redix emits, see `Redix.Telemetry`.","ref":"telemetry.html"},{"type":"extras","title":"Writing your own handler - Telemetry","doc":"If you want control on how Redix events are logged or on what level they're logged at, you can use your own event handler. For example, you can create a module to handle these events:\n\n    defmodule MyApp.RedixTelemetryHandler do\n      require Logger\n\n      def handle_event([:redix, event], _measurements, metadata, _config) do\n        case event do\n          :disconnection ->\n            human_reason = Exception.message(metadata.reason)\n            Logger.warn(\"Disconnected from #{metadata.address}: #{human_reason}\")\n\n          :failed_connection ->\n            human_reason = Exception.message(metadata.reason)\n            Logger.warn(\"Failed to connect to #{metadata.address}: #{human_reason}\")\n\n          :connection ->\n            Logger.debug(\"Connected/reconnected to #{metadata.address}\")\n        end\n      end\n    end\n\nOnce you have a module like this, you can attach it when your application starts:\n\n    events = [\n      [:redix, :disconnection],\n      [:redix, :failed_connection],\n      [:redix, :connection]\n    ]\n\n    :telemetry.attach_many(\n      \"my-redix-log-handler\",\n      events,\n      &MyApp.RedixTelemetryHandler.handle_event/4,\n      :config_not_needed_here\n    )\n\n[telemetry]: https://github.com/beam-telemetry/telemetry","ref":"telemetry.html#writing-your-own-handler"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.5.0 - Changelog","doc":"#","ref":"changelog.html#v1-5-0"},{"type":"extras","title":"New features - Changelog","doc":"* Add support for the `valkey://` scheme when using URIs.","ref":"changelog.html#new-features"},{"type":"extras","title":"v1.4.2 - Changelog","doc":"#","ref":"changelog.html#v1-4-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Speed up `Redix.Protocol` a little bit for common responses (`\"OK\"` and friends).\n  * Fix a bug where `:tcp_closed`/`:ssl_closed` and `:tcp_error`/`:ssl_error` messages wouldn't arrive to the socket owner, and Redix would get stuck in a disconnected state when sending would error out. See the discussion in [#265](https://github.com/whatyouhide/redix/issues/265).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.4.1 - Changelog","doc":"#","ref":"changelog.html#v1-4-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* `Redix.PubSub.get_client_id/1` is not available only behind the `:fetch_client_id_on_connect` option that you can pass to `Redix.PubSub.start_link/1`. This option defaults to `false`, so that this version of Redix is compatible with Redis v4 or earlier out of the box. To opt in into the behavior desired for [client-side caching](https://redis.io/docs/manual/client-side-caching/) and use `Redix.PubSub.get_client_id/1`, pass `fetch_client_id_on_connect: true` to `Redix.PubSub.start_link/1`.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.4.0 - Changelog","doc":"#","ref":"changelog.html#v1-4-0"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Introduce `Redix.PubSub.get_client/1`, which can be used to implement [client-side caching](https://redis.io/docs/manual/client-side-caching/).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.3.0 - Changelog","doc":"#","ref":"changelog.html#v1-3-0"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Improve EXITs that happen during calls to Redix functions.\n  * Remove call to deprecated `Logger.warn/2`.\n  * Support MFA for `:password` in the `:sentinel` option.\n  * Add the `Redix.password/0` type.\n  * Add the `Redix.sentinel_role/0` type.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.2.4 - Changelog","doc":"#","ref":"changelog.html#v1-2-4"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Remove Dialyzer PLTs from the Hex package. This has no functional impact whatsoever on the library. The PLTs were accidentally published together with the Hex package, which just results in an unnecessarily large Hex package.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.2.3 - Changelog","doc":"#","ref":"changelog.html#v1-2-3"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a bug with validating the `:socket_opts` option, which required a keyword list and thus wouldn't support *valid* options such as `:inet6`.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.2.2 - Changelog","doc":"#","ref":"changelog.html#v1-2-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Make parsing large bulk strings *a lot* faster. See [the pull request](https://github.com/whatyouhide/redix/pull/247) for benchmarks. This causes no functional changes, just a speed improvement.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.2.1 - Changelog","doc":"#","ref":"changelog.html#v1-2-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Relaxed the version requirement for the `:castore` dependency to support `~> 1.0`.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.2.0 - Changelog","doc":"#","ref":"changelog.html#v1-2-0"},{"type":"extras","title":"New features - Changelog","doc":"* Add `:telemetry_metadata` option to Redis calls. This can be used to provide custom metadata for Telemetry events.\n  * Mark **Redis sentinel** support as *not*-experimental anymore.\n  * Make `Redix.URI` part of the public API.\n\n#","ref":"changelog.html#new-features"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Handle Redis servers that disable the `CLIENT` command.\n  * Bump Elixir requirement to 1.11+.\n  * Raise an error if the `:timeout` option (supported by many of the function in the `Redix` module) is something other than a non-negative integer or `:infinity`. Before, `timeout: nil` was accidentally supported (but not documented) and would use a default timeout.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.1.5 - Changelog","doc":"#","ref":"changelog.html#v1-1-5"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix formatting of Unix domain sockets when logging\n  * Use `Logger` instead of `IO.warn/2` when warning about ACLs, so that it can be silenced more easily.\n  * Allow the `:port` option to be set explicitly to `0` when using Unix domain sockets\n  * Support empty string as database when using Redis URIs due to changes to how URIs are handled in Elixir","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.1.4 - Changelog","doc":"#","ref":"changelog.html#v1-1-4"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Support version 1.0 and over for the Telemetry dependency.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.1.3 - Changelog","doc":"#","ref":"changelog.html#v1-1-3"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* The `.formatter.exs` file included in this repo had some filesystem permission problems. This version fixes those.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.1.2 - Changelog","doc":"Version v1.1.1 was accidentally published with local code (from the maintainer's machine) in it instead of the code from the main Git branch. We're all humans! Version v1.1.1 has been retired.","ref":"changelog.html#v1-1-2"},{"type":"extras","title":"v1.1.1 - Changelog","doc":"#","ref":"changelog.html#v1-1-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Version v1.1.0 started using ACLs and issuing `AUTH    ` when a username was provided (either via options or via URI). This broke previous documented behavior, where Redix used to ignore usernames. With this bug fix, Redix now falls back to `AUTH  ` if `AUTH    ` fails because of the wrong number of arguments, which indicates a version of Redis earlier than version 6 (when ACLs were introduced).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"#","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Improve handling of databases in URIs.\n  * Add support for [ACL](https://redis.io/topics/acl), introduced in Redis 6.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"No bug fixes or improvements. Just enough years passed for this to become 1.0.0!","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.11.2 - Changelog","doc":"#","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a connection process crash that would very rarely happen when connecting to sentinel nodes with the wrong password or wrong database would fail to due a TCP/SSL connection issue.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.11.1 - Changelog","doc":"#","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Allow `nil` as a valid value for the `:password` start option again. v0.11.0 broke this feature.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"#","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Use the new Telemetry event conventions for pipeline-related events. The new events are `[:redix, :pipeline, :start]` and `[:redix, :pipeline, :stop]`. They both have new measurements associated with them.\n  * Remove the `[:redix, :reconnection]` Telemetry event in favor or `[:redix, :connection]`, which is emitted anytime there's a successful connection to a Redis server.\n  * Remove support for the deprecated `:log` start option (which was deprecated on v0.10.0).\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Add the `:connection_metadata` name to all connection/disconnection-related Telemetry events.\n  * Allow a `{module, function, arguments}` tuple as the value of the `:password` start option. This is useful to avoid password leaks in case of process crashes (and crash reports).\n  * Bump minimum Elixir requirement to Elixir `~> 1.7`.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.7 - Changelog","doc":"#","ref":"changelog.html#v0-10-7"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a crash in `Redix.PubSub` when non-subscribed processes attempted to unsubscribe.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.6 - Changelog","doc":"#","ref":"changelog.html#v0-10-6"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a bug that caused a memory leak in some cases for Redix pub/sub connections.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.5 - Changelog","doc":"#","ref":"changelog.html#v0-10-5"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix default option replacement for SSL in OTP 22.2.\n  * Allow `:gen_statem.start_link/3,4` options in `Redix.start_link/2` and `Redix.PubSub.start_link/2`.\n  * Change default SSL depth from 2 to 3 (see [this issue](https://github.com/whatyouhide/redix/issues/162)).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.4 - Changelog","doc":"#","ref":"changelog.html#v0-10-4"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix the default Telemetry handler for Redis Sentinel events (wasn't properly fixed in v0.10.3).\n  * Fix a compile-time warning about the [castore](https://github.com/elixir-mint/castore) library.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"#","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Use more secure SSL default options and optionally use [castore](https://github.com/elixir-mint/castore) if available as a certificate store.\n  * Fix the default Telemetry handler for Redis Sentinel events.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"#","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Allow a discarded username when using Redis URIs.\n  * Fix the `Redix.command/0` type which was `[binary()]` but which should have been `[String.Chars.t()]` since we call `to_string/1` on each command.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"#","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Improve password checking in Redis URIs.\n  * Fix a bug when naming Redix connections with something other than a local name.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"#","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Add support for [Telemetry](https://github.com/beam-telemetry/telemetry) and publish the following events: `[:redix, :pipeline]`, `[:redix, :pipeline, :error]`, `[:redix, :disconnection]`, `[:redix, :reconnection]`, `[:redix, failed_connection]`.\n  * Deprecate the `:log` option in `Redix.start_link/1` and `Redix.PubSub.start_link/1` in favour of Telemetry events and a default log handler that can be activated with `Redix.Telemetry.attach_default_handler/0`. See the documentation for [`Redix.Telemetry`](https://hexdocs.pm/redix/0.10.0/Redix.Telemetry.html). This is a hard deprecation that shows a warning. Support for the `:log` option will be removed in the next version.\n  * Fix a few minor bugs in `Redix.PubSub`.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.9.3 - Changelog","doc":"#","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a bug related to quickly reconnecting PIDs in `Redix.PubSub`.\n  * Improve error messages here and there.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"#","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Add support for URLs with the `rediss` scheme.\n  * Fix a bug where we used the wrong logging level in some places.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"#","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a bad return type from a `gen_statem` callback (#120).\n  * Improve logging for Redis Sentinel.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"#","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Bring `Redix.PubSub` into Redix. Pub/Sub functionality lived in a separate library, [redix_pubsub](https://github.com/whatyouhide/redix_pubsub). Now, that functionality has been moved into Redix. This means that if you use redix_pubsub and upgrade your Redix version to 0.9, you will use the redix_pubsub version of `Redix.PubSub`. If you also upgrade your redix_pubsub version, redix_pubsub will warn and avoid compiling `Redix.PubSub` so you can use the latest version in Redix. In general, if you upgrade Redix to 0.9 or later just drop the redix_pubsub dependency and make sure your application works with the latest `Redix.PubSub` API (the message format changed slightly in recent versions).\n\n  * Add support for Redis Sentinel.\n\n  * Don't raise `Redix.Error` errors on non-bang variants of functions. This means that for example `Redix.command/3` won't raise a `Redix.Error` exception in case of Redis errors (like wrong typing) and will return that error instead. In general, if you're pattern matching on `{:error, _}` to handle **connection errors** (for example, to retry after a while), now specifically match on `{:error, %Redix.ConnectionError{}}`. If you want to handle all possible errors the same way, keep matching on `{:error, _}`.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix a bug that wouldn't let you use Redis URIs without host or port.\n  * Don't ignore the `:timeout` option when connecting to Redis.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.8.2 - Changelog","doc":"#","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Fix an error when setting up SSL buffers (#106).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"#","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Re-introduce `start_link/2` with two lists of options, but deprecate it. It will be removed in the next Redix version.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"#","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Drop support for Elixir < 1.6.\n\n  * Unify `start_link` options: there's no more separation between \"Redis options\" and \"connection options\". Now, all the options are passed in together. You can still pass a Redis URI as the first argument. This is a breaking change because now calling `start_link/2` with two kewyord lists breaks. Note that `start_link/2` with two keyword lists still works, but emits a warning and is deprecated.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Rewrite the connection using [`gen_statem`](http://erlang.org/doc/man/gen_statem.html) in order to drop the dependency to [Connection](https://github.com/fishcakez/connection).\n\n  * Add `Redix.transaction_pipeline/3` and `Redix.transaction_pipeline!/3`.\n\n  * Use a timeout when connecting to Redis (which sometimes could get stuck).\n\n  * Add support for SSL 🔐\n\n  * Add `Redix.noreply_command/3` and `Redix.noreply_pipeline/3` (plus their bang `!` variants).","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"* Add support for Unix domain sockets by passing `host: {:local, path}`.","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"#","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Drop support for Elixir < 1.3.\n\n  * Remove `Redix.format_error/1`.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Add `Redix.child_spec/1` for use with the child spec changes in Elixir 1.5.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"* Fix some deprecation warnings around `String.to_char_list/1`.","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Start using `Redix.ConnectionError` when returning errors instead of just an atom. This is a breaking change since now `Redix.command/2` and the other functions return `{:error, %Redix.ConnectionError{reason: reason}}` instead of `{:error, reason}`. If you're matching on specific error reasons, make sure to update your code; if you're formatting errors through `Redix.format_error/1`, you can now use `Exception.message/1` on the `Redix.ConnectionError` structs.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"* Fix some TCP error handling during the connection setup phase.","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"* Fix `Redix.stop/1` to be synchronous and not leave zombie processes.","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"* Drop support for Elixir < 1.2 and OTP 17 or earlier.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* Add [@lexmag](https://github.com/lexmag) to the maintainers :tada:\n\n  * Handle timeouts nicely by returning `{:error, :timeout}` instead of exiting (which is the default `GenServer` behaviour).\n\n  * Remove support for specifying a maximum number of reconnection attempts when connecting to Redis (it was the `:max_reconnection_attempts` option).\n\n  * Use exponential backoff when reconnecting.\n\n  * Don't reconnect right away after the connection to Redis is lost, but wait for a cooldown time first.\n\n  * Add support for `:backoff_initial` and `:backoff_max` options in `Redix.start_link/2`. These options are used for controlling the backoff behaviour of a `Redix` connection.\n\n  * Add support for the `:sync_connect` option when connecting to Redis.\n\n  * Add support for the `:exit_on_disconnection` option when connecting to Redis.\n\n  * Add support for the `:log` option when connecting to Redis.\n\n  * Raise `ArgumentError` exceptions instead of `Redix.ConnectionError` exceptions for stuff like empty commands.\n\n  * Raise `Redix.Error` exceptions from `Redix.command/3` instead of returning them wrapped in `{:error, _}`.\n\n  * Expose `Redix.format_error/1`.\n\n  * Add a \"Reconnections\" page in the documentation.\n\n  * Extract the Pub/Sub functionality into [a separate project](https://github.com/whatyouhide/redix_pubsub).","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.6 - Changelog","doc":"* Fixed a bug in the integer parsing in `Redix.Protocol`.","ref":"changelog.html#v0-3-6"},{"type":"extras","title":"v0.3.5 - Changelog","doc":"* `Redix.Protocol` now uses continuations under the hood for a faster parsing experience.\n\n  * A bug in `Redix.Protocol` that caused massive memory leaks was fixed. This bug originated upstream in Elixir itself, and I submitted a fix for it [here](https://github.com/elixir-lang/elixir/pull/4350).\n\n  * Some improvements were made to error reporting in the Redix logging.","ref":"changelog.html#v0-3-5"},{"type":"extras","title":"v0.3.4 - Changelog","doc":"* Fix a bug in the connection that was replacing the provided Redis password with `:redacted` upon successful connection, making it impossible to reconnect in case of failure (because of the original password now being unavailable).","ref":"changelog.html#v0-3-4"},{"type":"extras","title":"v0.3.3 - Changelog","doc":"* Fix basically the same bug that was almost fixed in `v0.3.2`, but this time for real!","ref":"changelog.html#v0-3-3"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"* Fix a bug in the protocol that failed to parse integers in some cases.","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Restructure the Redix architecture to use two Elixir processes per connection instead of one (a process that packs commands and sends them on the socket and a process that listens from the socket and replies to waiting clients); this should speed up Redix when it comes to multiple clients concurrently issuing requests to Redis.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"#","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Change the behaviour for an empty list of command passed to `Redix.pipeline/2` (`Redix.pipeline(conn, [])`), which now raises a `Redix.ConnectionError` complaining about the empty command. Before this release, the behaviour was just a connection timeout.\n\n  * Change the behaviour of empty commands passed to `Redix.command/2` or `Redix.pipeline/2` (for example, `Redix.command(conn, [])` or `Redix.pipeline(conn, [[\"PING\"], []])`); empty commands now return `{:error, :empty_command}`. The previous behaviour was just a connection timeout.\n\n  * Remove `Redix.start_link/1` in favour of just `Redix.start_link/2`: now Redis options are separated from the connection options. Redis options can be passed as a Redis URI as well.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes and improvements - Changelog","doc":"* Change the error messages for most of the `Redix.ConnectionError` exceptions from simple atoms to more meaningful messages.","ref":"changelog.html#bug-fixes-and-improvements"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"* Fix a bug with single-element lists, that were parsed as single elements (and not lists with a single element in them) by `Redix.Protocol.parse_multi/2`. See [whatyouhide/redix#11](https://github.com/whatyouhide/redix/issues/11).","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* Rename `Redix.NetworkError` to `Redix.ConnectionError` (as it's more generic and more flexible).\n\n  * Add support for PubSub. The following functions have been added to the `Redix` module:\n    * `Redix.subscribe/4`\n    * `Redix.subscribe!/4`\n    * `Redix.psubscribe/4`\n    * `Redix.psubscribe!/4`\n    * `Redix.unsubscribe/4`\n    * `Redix.unsubscribe!/4`\n    * `Redix.punsubscribe/4`\n    * `Redix.punsubscribe!/4`\n    * `Redix.pubsub?/2`","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"License","doc":"The MIT License (MIT)\n\nCopyright (c) 2015 Andrea Leopardi\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}